@@ -22,6 +22,7 @@ import functools
 
 from googlecloudsdk.api_lib.compute import base_classes
 from googlecloudsdk.api_lib.compute import managed_instance_groups_utils
+from googlecloudsdk.api_lib.compute import utils
 from googlecloudsdk.api_lib.compute.instance_groups.managed import stateful_policy_utils as policy_utils
 from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.compute import flags
@@ -32,6 +33,7 @@ from googlecloudsdk.command_lib.compute.managed_instance_groups import auto_heal
 from googlecloudsdk.command_lib.util.apis import arg_utils
 import six
 
+
 # Flags valid only for regional MIGs.
 REGIONAL_FLAGS = [
     'instance_redistribution_type',
@@ -39,6 +41,15 @@ REGIONAL_FLAGS = [
     'on_repair_allow_changing_zone',
 ]
 
+INSTANCE_TEMPLATE_ARG = managed_flags.InstanceTemplateArg(
+    required=False,
+    help_override="""
+        Specifies the instance template that will replace the existing template used by the MIG.
+        The MIG then uses this new template to create instances. Specifying the new template overrides
+        any template versions that you've set for the MIG. Specify either a global or a regional instance template.
+        """,
+)
+
 
 # TODO(b/345166947) Remove universe annotation once b/341682289 is resolved.
 @base.UniverseCompatible
@@ -47,6 +58,8 @@ class UpdateGA(base.UpdateCommand):
   r"""Update a Compute Engine managed instance group."""
 
   support_update_policy_min_ready_flag = False
+  support_multi_mig_flag = False
+  support_instance_selection_min_cpu_platform = False
 
   @classmethod
   def Args(cls, parser):
@@ -71,15 +84,22 @@ class UpdateGA(base.UpdateCommand):
     managed_flags.AddMigInstanceRedistributionTypeFlag(parser)
     managed_flags.AddMigDistributionPolicyTargetShapeFlag(parser)
     managed_flags.AddMigListManagedInstancesResultsFlag(parser)
+    INSTANCE_TEMPLATE_ARG.AddArgument(parser)
     managed_flags.AddMigUpdatePolicyFlags(
         parser, support_min_ready_flag=cls.support_update_policy_min_ready_flag
     )
     managed_flags.AddMigForceUpdateOnRepairFlags(parser)
     managed_flags.AddMigDefaultActionOnVmFailure(parser, cls.ReleaseTrack())
     managed_flags.AddMigSizeFlag(parser)
-    managed_flags.AddInstanceFlexibilityPolicyArgs(parser, is_update=True)
+    flags.AddInstanceFlexibilityPolicyArgs(
+        parser,
+        is_update=True,
+        support_instance_selection_min_cpu_platform=cls.support_instance_selection_min_cpu_platform,
+    )
     managed_flags.AddStandbyPolicyFlags(parser)
     managed_flags.AddWorkloadPolicyFlags(parser)
+    if cls.support_multi_mig_flag:
+      managed_flags.AddMultiMigFlags(parser)
     # When adding RMIG-specific flag, update REGIONAL_FLAGS constant.
 
   def _GetUpdatedStatefulPolicyForDisks(
@@ -294,6 +314,13 @@ class UpdateGA(base.UpdateCommand):
     )
     return auto_healing_policies
 
+  def _GetInstanceTemplateRef(self, args, holder, default_scope):
+    return INSTANCE_TEMPLATE_ARG.ResolveAsResource(
+        args,
+        holder.resources,
+        default_scope=default_scope,
+    )
+
   def _PatchTargetDistributionShape(
       self,
       patch_instance_group_manager,
@@ -329,18 +356,36 @@ class UpdateGA(base.UpdateCommand):
       )
     return client.MakeRequests([(service, 'Patch', request)])
 
+  def _GetRegionForGroup(self, igm_ref):
+    if igm_ref.Collection() == 'compute.instanceGroupManagers':
+      return utils.ZoneNameToRegionName(igm_ref.zone)
+    else:
+      return igm_ref.region
+
   def _CreateInstanceGroupManagerPatch(
       self, args, igm_ref, igm_resource, client, holder
   ):
     """Create IGM resource patch."""
     managed_flags.ValidateRegionalMigFlagsUsage(args, REGIONAL_FLAGS, igm_ref)
     patch_instance_group_manager = client.messages.InstanceGroupManager()
+    include_fields = []
     auto_healing_policies = self._GetValidatedAutohealingPolicies(
         holder, client, args, igm_resource
     )
     if auto_healing_policies is not None:
       patch_instance_group_manager.autoHealingPolicies = auto_healing_policies
 
+    if args.IsSpecified('template'):
+      template_ref = self._GetInstanceTemplateRef(
+          args, holder, compute_scope.ScopeEnum.GLOBAL
+      )
+      patch_instance_group_manager.instanceTemplate = template_ref.SelfLink()
+      patch_instance_group_manager.versions = [
+          client.messages.InstanceGroupManagerVersion(
+              instanceTemplate=template_ref.SelfLink()
+          )
+      ]
+
     update_policy = managed_instance_groups_utils.PatchUpdatePolicy(
         client, args, igm_resource.updatePolicy
     )
@@ -370,10 +415,11 @@ class UpdateGA(base.UpdateCommand):
             client.messages, args
         )
     )
-    patch_instance_group_manager.instanceFlexibilityPolicy = (
-        managed_instance_groups_utils.CreateInstanceFlexibilityPolicy(
-            args, client.messages, igm_resource
-        )
+    patch_instance_group_manager.instanceFlexibilityPolicy = managed_instance_groups_utils.CreateInstanceFlexibilityPolicy(
+        args,
+        client.messages,
+        igm_resource,
+        support_instance_selection_min_cpu_platform=self.support_instance_selection_min_cpu_platform,
     )
     if args.IsSpecified('size'):
       patch_instance_group_manager.targetSize = args.size
@@ -385,9 +431,9 @@ class UpdateGA(base.UpdateCommand):
     )
     if standby_policy:
       patch_instance_group_manager.standbyPolicy = standby_policy
-    if args.suspended_size:
+    if args.IsKnownAndSpecified('suspended_size'):
       patch_instance_group_manager.targetSuspendedSize = args.suspended_size
-    if args.stopped_size:
+    if args.IsKnownAndSpecified('stopped_size'):
       patch_instance_group_manager.targetStoppedSize = args.stopped_size
 
     resource_policies = managed_instance_groups_utils.CreateResourcePolicies(
@@ -395,7 +441,20 @@ class UpdateGA(base.UpdateCommand):
     )
     patch_instance_group_manager.resourcePolicies = resource_policies
 
-    return patch_instance_group_manager
+    if args.IsKnownAndSpecified('multi_mig'):
+      if args.multi_mig:
+        multi_mig_region = self._GetRegionForGroup(igm_ref)
+        multi_mig_ref = holder.resources.Parse(
+            args.multi_mig,
+            params={'region': multi_mig_region, 'project': igm_ref.project},
+            collection='compute.regionMultiMigs',
+        )
+        patch_instance_group_manager.multiMig = multi_mig_ref.SelfLink()
+    elif args.IsKnownAndSpecified('clear_multi_mig'):
+      patch_instance_group_manager.multiMig = None
+      include_fields.append('multiMig')
+
+    return patch_instance_group_manager, include_fields
 
   def Run(self, args):
     holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
@@ -409,22 +468,32 @@ class UpdateGA(base.UpdateCommand):
         scope_lister=flags.GetDefaultScopeLister(client),
     )
 
-    if igm_ref.Collection() not in [
-        'compute.instanceGroupManagers',
-        'compute.regionInstanceGroupManagers',
-    ]:
-      raise ValueError(
-          'Unknown reference type {0}'.format(igm_ref.Collection())
+    if igm_ref:
+      collection = (
+          igm_ref[0].Collection()
+          if isinstance(igm_ref, list)
+          else igm_ref.Collection()
       )
+      if collection not in [
+          'compute.instanceGroupManagers',
+          'compute.regionInstanceGroupManagers',
+      ]:
+        raise ValueError('Unknown reference type {0}'.format(collection))
 
     igm_resource = managed_instance_groups_utils.GetInstanceGroupManagerOrThrow(
         igm_ref, client
     )
 
-    patch_instance_group_manager = self._CreateInstanceGroupManagerPatch(
-        args, igm_ref, igm_resource, client, holder
+    patch_instance_group_manager, include_fields = (
+        self._CreateInstanceGroupManagerPatch(
+            args, igm_ref, igm_resource, client, holder
+        )
     )
-    return self._MakePatchRequest(client, igm_ref, patch_instance_group_manager)
+
+    with client.apitools_client.IncludeFields(include_fields):
+      return self._MakePatchRequest(
+          client, igm_ref, patch_instance_group_manager
+      )
 
 
 UpdateGA.detailed_help = {
@@ -464,6 +533,8 @@ class UpdateBeta(UpdateGA):
   """Update a Compute Engine managed instance group."""
 
   support_update_policy_min_ready_flag = True
+  support_multi_mig_flag = True
+  support_instance_selection_min_cpu_platform = True
 
   @classmethod
   def Args(cls, parser):
@@ -474,12 +545,12 @@ class UpdateBeta(UpdateGA):
   def _CreateInstanceGroupManagerPatch(
       self, args, igm_ref, igm_resource, client, holder
   ):
-    patch_instance_group_manager = super(
+    patch_instance_group_manager, include_fields = super(
         UpdateBeta, self
     )._CreateInstanceGroupManagerPatch(
         args, igm_ref, igm_resource, client, holder
     )
-    return patch_instance_group_manager
+    return patch_instance_group_manager, include_fields
 
 
 UpdateBeta.detailed_help = UpdateGA.detailed_help
@@ -489,6 +560,9 @@ UpdateBeta.detailed_help = UpdateGA.detailed_help
 class UpdateAlpha(UpdateBeta):
   """Update a Compute Engine managed instance group."""
 
+  support_multi_mig_flag = True
+  support_instance_selection_min_cpu_platform = True
+
   @classmethod
   def Args(cls, parser):
     super(UpdateAlpha, cls).Args(parser)
@@ -496,11 +570,13 @@ class UpdateAlpha(UpdateBeta):
   def _CreateInstanceGroupManagerPatch(
       self, args, igm_ref, igm_resource, client, holder
   ):
-    igm_patch = super(UpdateAlpha, self)._CreateInstanceGroupManagerPatch(
+    igm_patch, include_fields = super(
+        UpdateAlpha, self
+    )._CreateInstanceGroupManagerPatch(
         args, igm_ref, igm_resource, client, holder
     )
 
-    return igm_patch
+    return igm_patch, include_fields
 
 
 UpdateAlpha.detailed_help = UpdateBeta.detailed_help
